#include <string.h>
#include <stdio.h>

#include "kmem.h"
#include "mem_layout.h"
#include "ptab.h"

static Map _map[2] =
{
    { 0x100000, static_cast<void (Root_page_table::*)(Address va, Address pa, Attr attr)> (&Pdir::map_1mb) },
    { 0x1000,   static_cast<void (Root_page_table::*)(Address va, Address pa, Attr attr)> (&Pdir::map_4kb) },
};

Map &
Pdir::get_map(unsigned index)
{
  if (index >= 2)
    {
      printf("Pdir: Warning: Invalid page size\n");
      while (1);
    }

  return _map[index];
}

Pdir::Pdir(Mword *raw)
  : _raw(raw)
{}

Address Pdir::phys() const
{
  return Mem_layout::virt_to_phys(reinterpret_cast<Address>(_raw));
}

void Pdir::map_1mb(Address va, Address pa, Attr attr)
{
  printf("pdir[%p]: map_1mb: %08lx -> %08lx\n", _raw, va, pa);
  Section &entry = reinterpret_cast<Section &>(_raw[index(va)]);
  
  if (entry.type != Type_invalid)
    {
      printf("Warning: entry is valid\n");
    }

  entry.set_base(pa);
  entry.type = Type_section;
 
  switch (attr)
    {
    case Kern:
      entry.ap = Section::Ap_kern;
      entry.cached = 1;
      break;
    
    case Device:
      entry.ap = Section::Ap_kern;
      break;
    
    case User:
      entry.ap = Section::Ap_user;
      entry.cached = 1;
      entry.local = 1;
      break;
    };
}

void Pdir::map_4kb(Address va, Address pa, Attr attr)
{
  printf("pdir[%p]: map_4kb: %08lx -> %08lx\n", _raw, va, pa);
  Coarse &entry = reinterpret_cast<Coarse &>(_raw[index(va)]);

  if (entry.type == Type_invalid)
    {
      Address base = reinterpret_cast<Address>(Kmem::page_alloc(Ptab::Size));
      entry.set_base(Mem_layout::virt_to_phys(base));
      entry.type = Type_coarse;
    }

  Ptab ptab(reinterpret_cast<Mword *>(Mem_layout::phys_to_virt(entry.get_base())));
  ptab.map(va, pa, attr);
}

//----------------------------------------------------------------------------

Ptab::Ptab(Mword *raw)
  : _raw(raw)
{}

void Ptab::map(Address va, Address pa, Attr attr)
{
  printf("ptab[%p]: map: %08lx -> %08lx\n", _raw, va, pa);
  Small &entry = reinterpret_cast<Small &>(_raw[index(va)]);

  if (entry.type == Type_invalid)
    {
      entry.set_base(pa);
      entry.type = Type_small;

      switch (attr)
	{
	case Kern:
	  entry.ap0 = Small::Ap_kern;
	  entry.ap1 = Small::Ap_kern;
	  entry.ap2 = Small::Ap_kern;
	  entry.ap3 = Small::Ap_kern;
	  entry.cached = 1;
	  break;

	case User:
	  entry.ap0 = Small::Ap_user;
	  entry.ap1 = Small::Ap_user;
	  entry.ap2 = Small::Ap_user;
	  entry.ap3 = Small::Ap_user;
	  entry.cached = 1;
	  break;

	case Device:
	  break;
	}
    }
  else
    {
      printf("Page table entry already exists\n");
    }
}

//----------------------------------------------------------------------------

#if 0
static unsigned Space_id = 1;

Space *Space::root()
{
  static Space _root;
  return &_root;
}

char root_pdir[sizeof(Pdir::Size)] __attribute__((aligned(0x4000), section(".data")));

Space::Space()
  : _asid(0), _pdir((Mword *)root_pdir)
{}

Space::Space(Space *space)
  : _asid(Space_id++), _pdir(Kmem::page_alloc(Pdir::Size))
{
  printf("[new space:%p] id:%d\n", this, _asid);
  copy_from(space, _pdir.index(Mem_layout::Phys_ram_base), _pdir.index(Mem_layout::Phys_ram_end));
  copy_from(space, _pdir.index(Mem_layout::Phys_device_base), _pdir.index(Mem_layout::Phys_device_end));
}

void
Space::init()
{
  Address va, pa;

  // map kernel ram 1:1
  for (va = Mem_layout::Phys_ram_base;
       va < Mem_layout::User_ram_base; va += 0x100000) 
    root()->map_1mb(va, va, Kern);
 
  // map device memory 1:1
  root()->map_1mb(Mem_layout::Phys_device_base,
                         Mem_layout::Phys_device_base, Device);
#if 0
  // map user ram 1:1
  for (va = Mem_layout::User_ram_base;
       va < Mem_layout::Phys_ram_end; va += 0x100000) 
    Space::root()->map_1mb(va, va, User);
#endif

  root()->activate();
}

void Space::map_1mb(Address va, Address pa, Attr attr)
{
  printf("space[%p]: map_1mb: %08lx -> %08lx\n", this, va, pa);
  _pdir.map_1mb(va, pa, attr);
}

void Space::map_4kb(Address va, Address pa, Attr attr)
{
  printf("space[%p]: map_4kb: %08lx -> %08lx\n", this, va, pa);
  _pdir.map_4kb(va, pa, attr);
}

void Space::copy_from(const Space *space, unsigned idx_start, unsigned idx_end)
{
  for (unsigned i = idx_start; i < idx_end; ++i)
    {
      _pdir.set_entry(i, space->pdir().get_entry(i));
    }
}

#endif
