#pragma once

#include "types.h"
#include "ptab_attr.h"

/**
 * Encapsulates a page directory
 */

class Pdir : public Root_page_table
{
  /**
   * Sizes of possible mappings
   */
  enum
    {
      Type_4kb	= 0x0,
      Type_4mb	= 0x1,
    };

  /**
   * Describes a page directory entry
   * (see Intel manual for details!)
   */
  struct Entry
    {
      // XXX implement me!

      /**
       * Returns the physical base address.
       * \return	physical base address
       */
      Address get_base();
      
      /**
       * Sets a new physical base address.
       * \param pa	physical address
       */
      void set_base(Address pa);
    };

public:
  
  enum
    {
      Shift	= 22,
      Size	= 0x1000,
    };
  
  Pdir(Mword *raw);

  /**
   * Return physical address of page directory
   */
  Address phys() const;

  /**
   * Return entry from page directory
   * \param index	index of the entry
   */
  Mword get_entry(unsigned index) const
    { return _raw[index]; }
  /**
   * set entry in page directory
   * \param index	index of the entry
   * \param entry	new entry
   */
  void set_entry(unsigned index, Mword entry)
    { _raw[index] = entry; }

  /**
   * Get index from virtual index
   * \param va		virtual address
   * \returns		index
   */
  unsigned index(Address va)
    { return va >> Shift; }

  /**
   * Establish a virtual to physical 4MB mapping in the page directory.
   *
   * \param va		virtual address
   * \param pa		physical address
   * \param attr	attribute of the mapping
   */
  void map_4mb(Address va, Address pa, Attr attr);
  /**
   * Establish a virtual to physical 4KB mapping in the page directory.
   *
   * \param va		virtual address
   * \param pa		physical address
   * \param attr	attribute of the mapping
   */
  void map_4kb(Address va, Address pa, Attr attr);
 
  /**
   * Get a reference to a mapping function
   *
   * \param index	size that should be mapped(i.e large or small)
   * \return		reference to mapping function
   */
  virtual Map &get_map(unsigned index);

private:
  Mword *_raw;
};

//----------------------------------------------------------------------------

/**
 * Encapsulates a page table
 */

class Ptab
{
  /**
   * Describes a page table entry
   * (see Intel manual for details!)
   */
  struct Entry
    {
      // XXX implement me!

      /**
       * Sets a new physical base address.
       * \param pa	physical address
       */
      void set_base(Address pa);
    };

public:
  
  enum
    {
      Shift	= 12,		///< Bits to shift a virtual address to create the index
      Mask	= 0xff,		///< 
      Size	= 0x1000,	///< Size of a page table
    };

  /**
   * Constructor that creates an empty page table
   * \param raw		pointer to the memory
   */
  Ptab(Mword *raw);

  /**
   * Establish a virtual to physical mapping in the page table.
   *
   * \param va		virtual address
   * \param pa		physical address
   * \param attr	attribute of the mapping
   */
  void map(Address va, Address pa, Attr attr);

private:
  /**
   * Return index into page table for virtual address
   * \param va		virtual address
   */
  unsigned index(Address va)
    { return (va >> Shift) & Mask; }
  
  Mword *_raw;
};
