#
# Rules for building a library target
#
# The following variables must be passed when calling this file:
#
#   BASE_DIR         - base directory of the build system
#   REPOSITORIES     - repositories providing libs and headers
#   VERBOSE          - build verboseness modifier
#   VERBOSE_DIR      - verboseness modifier for changing directories
#   VERBOSE_MK       - verboseness of make calls
#   BUILD_BASE_DIR   - base of build directory tree
#   LIB_CACHE_DIR    - library build cache location
#   LIB_PROGRESS_LOG - library build log file
#
#$(info 'lib.mk REPOSITORIES:   $(REPOSITORIES)\n')
#$(info 'lib.mk BASE_DIR:       $(BASE_DIR)\n')
#$(info 'lib.mk BUILD_BASE_DIR: $(BUILD_BASE_DIR)\n')
#$(info 'lib.mk LIB_CACHE_DIR:  $(LIB_CACHE_DIR)\n')

#
# Function that searches for files in all
# repositories and returns the first match.
#
select_from_repositories = $(firstword $(foreach REP,$(REPOSITORIES),$(wildcard $(REP)/$(1))))

#
# Read SPECS configuration
#
-include $(call select_from_repositories,etc/specs.conf)
-include $(BUILD_BASE_DIR)/etc/specs.conf

#
# The target name corresponds to the basename of the .lib.o file
#
TARGET = $(subst .lib.o,,$(notdir $(LIB_O_TARGET)))

#
# Prefix all object files with the library name
#
OBJ_PREFIX = $(TARGET)_

#
# Include specifics, for example platform, kernel-api etc.
#
include $(foreach SPEC,$(SPECS),$(call select_from_repositories,mk/spec-$(SPEC).mk))

#
# Search source repositories for matching libname.mk file
#
# First, we determine all possible locations where the libname.mk
# file could reside. These locations depend on the repositories and
# the SPECS.
#
#LIB_MK_DIRS = $(foreach REP,$(REPOSITORIES),$(addprefix $(REP)/lib/mk/,$(SPECS) .))
LIB_MK_DIRS = $(foreach REP,$(REPOSITORIES),$(shell find $(REP) -name $(TARGET) 2>/dev/null))

#
# Of all possible file locations, use the (first) one that actually exist.
#
LIB_MK_FILE = $(firstword $(wildcard $(addsuffix /$(TARGET).mk,$(LIB_MK_DIRS))))

#
# Sanity check to detect missing library description file
#
ifeq ($(LIB_MK_FILE),)
done:
	@echo "Skip $(TARGET) library because of missing library description file $(TARGET).mk"
endif

#
# Determine the repository base directory from the absolute pathname
# of the choosen libname.mk file. We need to specify the override
# command because REP_DIR was first set by prg.mk when building a
# program target. The repository of a library could be a different
# one.
#
override REP_DIR := $(firstword $(foreach REP,$(REPOSITORIES),$(findstring $(REP)/,$(LIB_MK_FILE))))

#
# Prevent <libname>.mk rules to be executed as default rule
#
all::

#
# Include library build instructions
#
include $(LIB_MK_FILE)

#
# Check if the requirements of the target are satisfied
#
UNSATISFIED_REQUIREMENTS = $(filter-out $(SPECS),$(REQUIRES))
ifneq ($(UNSATISFIED_REQUIREMENTS),)
stop_here:
	@$(ECHO) "Skip library $(TARGET) because it requires $(DARK_COL)$(UNSATISFIED_REQUIREMENTS)$(DEFAULT_COL)"
endif

include $(BASE_DIR)/mk/global.mk

#
# Build the libraries we depend on
#
ifneq ($(LIBS),)
all::
	$(VERBOSE_MK)$(MAKE) $(VERBOSE_DIR)  -f $(BASE_DIR)/mk/build_libs.mk \
	  BASE_DIR=$(BASE_DIR) \
	  BUILD_BASE_DIR=$(BUILD_BASE_DIR) \
	  TARGET_DIR=$(TARGET_DIR) \
	  REPOSITORIES="$(REPOSITORIES)" \
	  VERBOSE=$(VERBOSE) \
	  VERBOSE_DIR=$(VERBOSE_DIR) \
	  VERBOSE_MK=$(VERBOSE_MK) \
	  LIB_CACHE_DIR=$(LIB_CACHE_DIR) \
	  LIB_PROGRESS_LOG=$(LIB_PROGRESS_LOG) \
	  $(addsuffix .lib.o,$(LIBS))
endif

#
# Name of <libname>.lib.o file to create
#
LIBOBJ = $(addsuffix .lib.o,$(TARGET))

#
# Build <libname>.lib.o target only if it is
# not already contained in the library build log
#
include $(LIB_PROGRESS_LOG)
ifeq ($(filter $(LIBOBJ),$(LIBS_READY)),)

#
# Print message for the currently built library
#
all::
	$(MSG_LIB)$(TARGET)

#
# Create <libname>.lib.o file.
#
all:: $(LIBOBJ)

include $(BASE_DIR)/mk/generic.mk

#
# Rule to build the <libname>.lib.o file
#
$(LIBOBJ):: $(OBJECTS)
	$(MSG_MERGE)$(LIBOBJ)
	$(VERBOSE)$(LD) -r $^ -o $@

endif

#
# Include built library in library build log
# If library is already contained in the log,
# do something (true) to avoid a message by
# make.
#
all::
ifeq ($(filter $(LIBOBJ),$(LIBS_READY)),)
	@echo "LIBS_READY += $(LIBOBJ)" >> $(LIB_PROGRESS_LOG)
else
	@true
endif
